const config = require('../../../config/config')

const {
  createRule: blockchainCreateRule,
  updateRule: blockchainUpdateRule,
  deleteRule: blockchainDeleteRule,
  getRules: blockchainGetRules,
  getPatientAccessRead: blockchainGetPatientAccessRead,
  getPatientAccessAllRead: blockchainGetPatientAccessAllRead,
  getAccessLog: blockchainGetAccessLog,
  ping: blockchainPing,
  reset: blockchainReset,
  createProof: blockchainCreateProof,
  updateProof: blockchainUpdateProof,
  getProof: blockchainGetProof
} = require('../../../services/chaincode/contracts/serumsContract')(config.blockchain)

const logger = require('../../../utils/logging/logger').getLogger('api/REST/serumsController')

function serumsController () {
  /*
        Creates a rule giving access to patient data to a group of people, or to an indidual
        Body: The rule
        Returns: The ruleId
    */
  function createRule (req, res, next) {
    const requestBody = req.body

    logger.info(`Creating rule: ${JSON.stringify(requestBody)}`)

    return blockchainCreateRule(res.locals.requestorOrgId, res.locals.jwtToken, requestBody)
      .then((result) => res.json(result))
      .catch((err) => {
        logger.error(`An error occured while creating a rule: ${err.message}`, { error: err })
        next(err)
      })
  }

  /*
        Updates a rule giving access to patient data to a group of people, or to an indidual
        Body: The rule
        Params: RuleId
        Returns: The ruleId
    */
  function updateRule (req, res, next) {
    const { ruleId } = req.params
    const requestBody = req.body

    // Add the requestor organisation id to the grantor
    requestBody.grantor.orgId = res.locals.requestorOrgId

    logger.info(`Updating rule: ${JSON.stringify(requestBody)}`)

    return blockchainUpdateRule(res.locals.requestorOrgId, res.locals.jwtToken, requestBody, ruleId)
      .then((result) => res.json(result))
      .catch((err) => {
        logger.error(`An error occured while updating a rule: ${err.message}`, { error: err })
        next(err)
      })
  }

  /*
        Deletes a rule giving access to patient data to a group of people, or to an indidual
        Params: RuleId
        Returns: Success
    */
  function deleteRule (req, res, next) {
    const { ruleId } = req.params
    logger.info(`Deleting rule ${ruleId}`)

    return blockchainDeleteRule(res.locals.requestorOrgId, res.locals.jwtToken, ruleId)
      .then((result) => res.json(result))
      .catch((err) => {
        logger.error(`An error occured while deleting a rule: ${err.message}`, { error: err })
        next(err)
      })
  }

  /*
        Gets the rules giving access to patient data to a group of people, or to an indidual
        Body: (optional) Filters
        Returns: List of rules
    */
  function getRules (req, res, next) {
    // Limitted to the organisation or to the patient
    const requestBody = req.body
    logger.info(`Getting rules: ${JSON.stringify(requestBody)}`)

    return blockchainGetRules(res.locals.requestorOrgId, requestBody)
      .then((result) => res.json(result))
      .catch((err) => {
        logger.error(`An error occured while getting the rules: ${err.message}`, { error: err })
        next(err)
      })
  }

  function getOneRule (req, res, next) {
    const { ruleId } = req.params
    logger.info(`Getting rule ${JSON.stringify(ruleId)}`)

    return blockchainGetRules(
      res.locals.requestorOrgId,
      {
        filters: [{
          key: 'id',
          value: ruleId,
          filterType: 'SIMPLE'
        }]
      }
    )
      .then((result) => res.json(result))
      .catch((err) => {
        logger.error(`An error occured while getting the rules: ${err.message}`, { error: err })
        next(err)
      })
  }

  /*
        Asks the blockchain if the caller has access to some patient data
        Body: Request details
        Returns: A vault token if successful
    */
  function getPatientAccess (req, res, next) {
    const requestBody = req.body
    logger.info(`Requesting patient access: ${JSON.stringify(requestBody)}`)
    if (requestBody.access) {
      logger.info('Requesting access read')
      return blockchainGetPatientAccessRead(res.locals.requestorOrgId, res.locals.jwtToken, requestBody)
        .then((blockchainResponse) => {
          logger.info(`Got response from blockchain: ${JSON.stringify(blockchainResponse)}`)
          if (blockchainResponse.accessGranted) {
            return res.json({
              accessGranted: true,
              access: blockchainResponse.access,
              ruleIds: blockchainResponse.ruleIds
            })
          }

          return res.json({ accessGranted: false })
        })
    }
    logger.info('Requesting access readall')
    return blockchainGetPatientAccessAllRead(res.locals.requestorOrgId, res.locals.jwtToken, requestBody)
      .then((blockchainResponse) => {
        logger.info(`Got response from blockchain: ${JSON.stringify(blockchainResponse)}`)
        if (blockchainResponse.accessGranted) {
          return res.json({
            accessGranted: true,
            access: blockchainResponse.access
          })
        }
        return res.json({ accessGranted: false })
      })
      .catch((err) => {
        logger.error(`An error occured while requesting patient access: ${err.message}`, { error: err })
        next(err)
      })
  }

  /*
        Asks the blockchain access logs for some patient
        Body: (optional) Filters
        Returns: Access logs
    */
  function getAccessLog (req, res, next) {
    const requestBody = req.body
    logger.info(`Requesting access log: ${JSON.stringify(requestBody)}`)

    return blockchainGetAccessLog(res.locals.requestorOrgId, requestBody)
      .then((result) => {
        logger.info(`Got access log of ${JSON.stringify(result)}`)
        res.json(result)
      })
      .catch((err) => {
        logger.error(`An error occured while requesting the access log: ${err.message}`, { error: err })
        next(err)
      })
  }

  /*
        Pings the blockchain
        Returns: Pong
    */
  function pingRequest (req, res, next) {
    logger.info('Pinging the blockchain...')
    return blockchainPing(config.serums.orgs[0])
      .then((result) => res.json(result))
      .catch((err) => {
        logger.error(`An error occured while getting executing ping: ${err.message}`, { error: err })
        next(err)
      })
  }

  /**
    *   Resets all of the rules and access logs
    */
  function resetRequest (req, res, next) {
    logger.info('Resetting the blockchain database...')
    return blockchainReset(config.serums.orgs[0])
      .then((result) => res.json(result))
      .catch((err) => {
        logger.error(`An error occured while getting executing reset: ${err.message}`, { error: err })
        next(err)
      })
  }

  function createProof (req, res, next) {
    const requestBody = req.body

    logger.info(`Creating proof: ${JSON.stringify(requestBody)}`)

    return blockchainCreateProof(res.locals.requestorOrgId, requestBody)
      .then((result) => res.json(result))
      .catch((err) => {
        logger.error(`An error occured while creating a proof: ${err.message}`, { error: err })
        next(err)
      })
  }

  function updateProof (req, res, next) {
    const { proofId } = req.params
    const requestBody = req.body

    logger.info(`Updating proof: ${JSON.stringify(requestBody)}`)

    return blockchainUpdateProof(res.locals.requestorOrgId, requestBody, proofId)
      .then((result) => res.json(result))
      .catch((err) => {
        logger.error(`An error occured while updating a proof: ${err.message}`, { error: err })
        next(err)
      })
  }

  function getProof (req, res, next) {
    const { proofId } = req.params
    logger.info(`Getting proof: ${proofId}`)

    return blockchainGetProof(res.locals.requestorOrgId, proofId)
      .then((result) => res.json(result[0]))
      .catch((err) => {
        logger.error(`An error occured while getting the proof: ${err.message}`, { error: err })
        next(err)
      })
  }

  return {
    createRule,
    updateRule,
    deleteRule,
    getRules,
    getOneRule,
    getPatientAccess,
    getAccessLog,
    pingRequest,
    resetRequest,
    createProof,
    updateProof,
    getProof
  }
}

module.exports = serumsController
