const config = require('../../config/config')

const logger = require('../../utils/logging/logger').getLogger('middleware/validateRequest')

const createError = require('../../utils/error/createError')
const { validateJWT, validateStaticJWT } = require('../../services/authentication/authenticationService')(config.auth)
const { NOT_AUTHENTICATED, NOT_AUTHORIZED, INVALID_TOKEN, AUTHENTICATION_ERROR } = require('../../constants/errorKeys')
const { setConfigSetting } = require('fabric-client')

const BEARER_TOKEN_REGEX = /^Bearer (.*)$/i

/**
 * Middleware to validate and set the user context
 * Returns an unauthorized error when the user is not authenticated and the url is not specified as unsecure
 * @param {Object} router Router to add middleware for
 * @param {Object} options Extra options to pass, eg public urls which don't require authentication
 */
module.exports = function validateRequest (router, { publicUrls = [], basicAuthorization = [] } = {}) {
  // Validate if the user is logged in
  router.use(async (req, res, next) => {
    const requiresNoAuthentication = publicUrls.find(
      (item) => item.url.toLowerCase() === req.url.toLowerCase() && item.method === req.method
    )
    const requiresBasicAuthentication = basicAuthorization.find(
      (item) => (item.url.toLowerCase() === req.url.toLowerCase() && item.method === req.method) || (req.url.match(/proof/) && true)
    )

    if (requiresNoAuthentication) {
      return next()
    }

    // Check if authorization headers are present
    if (!req.headers || !req.headers.authorization) {
      return next(createError(401, 'Authorization header not found', NOT_AUTHENTICATED))
    }

    const requestIsGetRules = req.url === '/getRules'
    const noFilterProvided = !req.body.filters || !req.body.filters[0] || !req.body.filters[0].filterType
    if (requestIsGetRules && noFilterProvided) {
      return next(createError(400, 'No filter provided', 'NO_FILTER'))
    }

    const authorizationHeader = req.headers.authorization
    logger.debug(`Authorization: ${authorizationHeader}`)

    if (requiresBasicAuthentication ) {
      const authenticated = await validateStaticJWT(authorizationHeader)
      if (authenticated) {
        // TODO do we always need and ORG-ID, kann we keep the same one here always?
        res.locals = {
          requestorOrgId: 'USTAN'
        }
        return next()
      }
      return next(createError(401, 'Authorization failed: Accesstoken is invalid!', 'NOT_AUTHORIZED'))
    }

    // Check if authorization header has a Bearer Token inside
    if (!BEARER_TOKEN_REGEX.test(authorizationHeader)) {
      return next(createError(400, 'Invalid token - Bearer not found', INVALID_TOKEN))
    }

    const jwtToken = authorizationHeader.match(BEARER_TOKEN_REGEX)[1]

    let authenticationResponse

    try {
      authenticationResponse = await validateJWT(jwtToken)
    } catch (err) {
      return next(createError(500, 'Authorization failed, please try again later', AUTHENTICATION_ERROR))
    }

    logger.info(`UA response on token verification: ${authenticationResponse}`)

    if (authenticationResponse.status !== 200) {
      logger.error(`Error received: ${authenticationResponse.message}`)
      return next(createError(401, 'Authorization failed: Accesstoken is invalid!', NOT_AUTHORIZED))
    }

    res.locals = {
      jwtToken,
      requestorOrgId: authenticationResponse.orgID,
      requestorGroups: authenticationResponse.groupIDs,
      requestorIndividualId: authenticationResponse.username
    }

    logger.debug(res.locals)

    // Check if requester ORG is known
    if (!config.serums.orgs.includes(res.locals.requestorOrgId)) {
      logger.error(`Requestor id ${res.locals.requestorOrgId} is not managed by this backend: ${config.serums.orgs}`)
      return next(createError(401, 'Unknown requester ORG', NOT_AUTHORIZED))
    }

    return next()
  })
}
