const log4js = require('log4js')
const fs = require('fs-extra')

const logFilePath = process.env.LOG_LOCATION
const appenders = {
    out: {type: 'stdout'}
}

function getLogLevel(level) {
    let logLevel = 'debug'

    if (typeof level === 'string') {
        switch (level.toUpperCase()) {
            case 'OFF':
                logLevel = 'off'
                break
            case 'CRITICAL':
                logLevel = 'fatal'
                break
            case 'ERROR':
                logLevel = 'error'
                break
            case 'WARNING':
                logLevel = 'warn'
                break
            case 'DEBUG':
                logLevel = 'debug'
                break
            default:
                logLevel = 'info'
        }
    }
    return logLevel
}

const logLevel = getLogLevel(process.env.APP_LOGGING_LEVEL)

if (logFilePath) {
    // Clean old logs
    fs.removeSync(logFilePath)

    fs.ensureFileSync(logFilePath)

    appenders.fileLog = {
        type: 'file',
        filename: logFilePath,
        maxLogSize: process.env.LOG_SIZE || 10485760, // the maximum size (in bytes) for the log file (10MB)
        backups: process.env.LOG_BACKUPS || 5, // the number of old log files to keep during log rolling.
        keepFileExt: true
    }
}

log4js.configure({
    appenders,
    categories: {
        default: {
            appenders: Object.keys(appenders),
            level: logLevel
        }
    }
})

const mainLogger = log4js.getLogger('serums-backend/src/utils/logging/logger.js')
mainLogger.level = 'info'

mainLogger.info(`Log level is set to ${logLevel}`)


module.exports.getLogger = (name) => {
    // set the logging level based on the environment variable

    const logger = log4js.getLogger(name)
    logger.level = logLevel

    function newLog(logMethod, msg, object = {}) {
        if (typeof object !== 'object') {
            logger.warn(`The additional data when logging must be an object, but received a ${typeof object}`)
            logMethod(`${msg}`)
        }
        if (Object.keys(object).length > 0) {
            logMethod.call(logger, `${msg}. ${JSON.stringify(object)}`)
        } else {
            logMethod.call(logger, msg)
        }
    }

    function trace(msg, object = {}) {
        newLog(logger.trace, msg, object)
    }

    function debug(msg, object = {}) {
        newLog(logger.debug, msg, object)
    }

    function info(msg, object = {}) {
        newLog(logger.info, msg, object)
    }

    function warn(msg, object = {}) {
        newLog(logger.warn, msg, object)
    }

    function error(msg, object = {}) {
        newLog(logger.error, msg, object)
    }

    function fatal(msg, object = {}) {
        newLog(logger.fatal, msg, object)
    }

    return {
        trace,
        debug,
        info,
        warn,
        error,
        fatal
    }
}
