const {
    Chaincode,
    ChaincodeError,
    NotFoundError,
    Helpers
} = require('@theledger/fabric-chaincode-utils')

const jwt = require('jsonwebtoken')
const Yup = require('yup')

const {
    rule: ruleShape,
    ruleId: ruleIdShape,
    proofId: proofIdShape,
    accessRequest: accessRequestShape,
    accessLogFilters: accessLogFiltersShape,
    getRulesFilters: getRulesFiltersShape,
    ruleIdRegex,
    jwtToken: jwtTokenShape,
    rule,
    create_proof: createProofShape,
    update_proof: updateProofShape
} = require('./shapes')



const serumsAuthConfig = {
    cert: 'myPublicKey',
    issuers: ['SerumsAuthentication']
}

class User {
    constructor({orgId, groupIds, individualId, departmentIds}) {
        this.orgId = orgId
        this.groupIds = groupIds
        this.individualId = individualId
        this.departmentIds = departmentIds
    }

    /*
        Returns a rule managed by this user
    */
    ruleForUser(rule) {
        const grantorId = rule.grantor.id || this.individualId
        return {
            ...rule,
            grantor: {
                ...rule.grantor,
                id: grantorId,
                orgId: this.orgId
            }
        }
    }

    /**
    *   Returns True if this user can manage this rule
    */
    canManageRule(rule) {
        // Must be of same organisation
        let canManageRule = rule.grantor.orgId === this.orgId
        const isAdmin = this.groupIds.includes('HOSPITAL_ADMIN')
        // Double-equals here is important as the individualId is received as INT and grantorId is STRING
        const grantorIsCaller = rule.grantor.id == this.individualId

        if (rule.grantor.type === 'INDIVIDUAL') {
            // If rule is for a specific individual, only him can manage this rule, except if he's an admin
            canManageRule = canManageRule && (isAdmin || grantorIsCaller)
        } else {
            // This rule gives access to all patients of this hospital, so the user must be an admin for that hospital
            canManageRule = canManageRule && isAdmin && grantorIsCaller
        }

        return canManageRule
    }

    /**
    *   Throws an error if this user cannot manage this rule
    */
    checkCanManageRule(rule) {
        if (!this.canManageRule(rule)) {
            throw new ChaincodeError(`User ${JSON.stringify({
                orgId: this.orgId,
                groupIds: this.groupIds,
                individualId: this.individualId,
                departmentIds: this.departmentIds
            })} cannot manage this rule.`)
        }
    }
}

function getQueryForAccessRequest(stubHelper, requestor, target) {
    const query = {
        selector: {
            $and: [
                {
                    // We're only matching on rules.
                    id: {
                        $regex: ruleIdRegex.source
                    },
                    // grantee id must match the requestor orgId
                    'grantee.orgId': requestor.orgId,
                    // grantor id must match the target orgId
                    'grantor.orgId': target.patientOrgId,
                    'expires': {
                        $gte: stubHelper.getTxDate().toISOString()
                    }
                },
                {
                    // The requestor must match
                    $or: [
                        // Match on individualId
                        {
                            'grantee.id': {
                                $eq: requestor.individualId
                            },
                            'grantee.type': 'INDIVIDUAL'
                        },
                        // Match on groupId
                        {
                            'grantee.id': {
                                $in: requestor.groupIds
                            },
                            'grantee.type': 'GROUP'
                        },
                        {
                            'grantee.id': {
                                $in: requestor.departmentIds
                            },
                            'grantee.type': 'DEPARTMENT'
                        }
                    ]
                },
                {
                    // The target must match
                    $or: [
                        // Match on individualId
                        {
                            'grantor.id': {
                                $eq: target.patientId
                            },
                            'grantor.type': 'INDIVIDUAL'
                        },
                        // Or the grantor is a group, in which case it's a hospital
                        {
                            'grantor.type': 'GROUP'
                        }
                    ]
                }
            ]
        }
    }

    if (target.access) {
        query.selector['$and'].push({
            // The rule must apply to at least one of the access attributes
            access: {
                $elemMatch: {
                    $or: target.access
                }
            }
        })
    }
    return query
}

function removeDuplicates(myArr, prop) {
    return myArr.filter((obj, pos, arr) => {
        return arr.map(mapObj => mapObj[prop]).indexOf(obj[prop]) === pos;
    });
}

async function decodeToken(stubHelper, rawToken) {
    try {
        // Check argument type
        const token = await Helpers.checkArgs(rawToken, jwtTokenShape)
        // Check signature
        const decodedToken = jwt.decode(token)

        return {
            orgId: decodedToken.orgID,
            groupIds: decodedToken.groupIDs,
            individualId: decodedToken.userID,
            departmentIds: decodedToken.departmentIDs,
            expiration: new Date(decodedToken.exp)
        }
    }

    catch (error) {
        const msg = `Got an error verifying the token: ${error.message}`
        throw new ChaincodeError(msg)
    }
}

class SerumsChaincode extends Chaincode {
    /**
    *   Pings the blockchain
    *   PUBLIC
    */
    async ping() {
        this.logger.info('ping called', '')
        return {ping: 'pong'}
    }

    /**
    *   Creates a rule
    *   PERMISSIONED
    */
    async createRule(stubHelper, args) {
        const decodedToken = await decodeToken(stubHelper, args[0])
        const user = new User(decodedToken)
        let rule = await Helpers.checkArgs(args[1], Yup.object().shape(ruleShape).required())

        let now = new Date()
        rule.created_at = now.toISOString()

        if (rule.requestor == undefined) {
            rule.requestor = decodedToken.userID
        }

        // Make rule for user
        rule = user.ruleForUser(rule)
        this.logger.info(`Created user from rule: ${JSON.stringify(user)}`)
        user.checkCanManageRule(rule)

        rule.id = `RULE_${stubHelper.generateUUID('RULE')}`

        // Convert date to toISOString
        rule.expires = rule.expires.toISOString()

        await stubHelper.putState(rule.id, rule)
        this.logger.info(`Created rule ${rule.id}`)

        return {ruleId: rule.id}
    }

    /**
    *   Updates an existing rule
    *   PERMISSIONED
    */
    async updateRule(stubHelper, args) {
        const decodedToken = await decodeToken(stubHelper, args[0])
        const user = new User(decodedToken)

        let rule = await Helpers.checkArgs(args[1], Yup.object().shape(ruleShape).required())
        const ruleId = await Helpers.checkArgs(args[2], ruleIdShape.ensure())

        // Make rule for user
        rule = user.ruleForUser(rule)
        // Check the user owns rights over this rule
        user.checkCanManageRule(rule)

        rule.id = ruleId

        // Convert date to toISOString
        rule.expires = rule.expires.toISOString()

        // Check if previous rule exists
        const previousRule = await stubHelper.getStateAsObject(ruleId)
        if (!previousRule) {
            throw new ChaincodeError(`Rule with id ${ruleId} does not exist.`, 404)
        }
        // Check the user can modify this rule
        user.checkCanManageRule(previousRule)

        // Override previous rule
        await stubHelper.putState(ruleId, rule)
        this.logger.info(`Updated rule ${ruleId}`)
        return {ruleId}
    }

    /**
    *   Deletes an existing rule
    *   PERMISSIONED
    */
    async deleteRule(stubHelper, args) {
        const decodedToken = await decodeToken(stubHelper, args[0])
        const user = new User(decodedToken)
        const ruleId = await Helpers.checkArgs(args[1], ruleIdShape.required())

        const previousRule = await stubHelper.getStateAsObject(ruleId)
        if (!previousRule) {
            throw new ChaincodeError(`Rule with id ${ruleId} does not exist.`, 404)
        }
        // Check the user can modify this rule
        user.checkCanManageRule(previousRule)

        await stubHelper.deleteAllByQuery({
            selector: {
                id: {
                    $eq: ruleId
                }
            },
            fields: ['id']
        })

        this.logger.info(`Deleted rule ${ruleId}`)

        return {success: true}
    }

    /**
    *   Get rules
    *   PUBLIC
    */
    async getRules(stubHelper, args) {
        const {filters} = await Helpers.checkArgs(args[0] || {filters: []}, Yup.object().shape(getRulesFiltersShape).required())
        this.logger.info('Getting rules')

        const filterSelect = filters.map((f) => {
            const selector = {}

            if (f.filterType == 'SIMPLE') {
                selector[f.key] = {
                    $eq: f.value
                }
            } else if (f.filterType == 'NOT_EXPIRED') {
                selector['expires'] = {
                    $gte: stubHelper.getTxDate().toISOString()
                }
            } else {
                this.logger.info(`Unknown filterType: ${f.filterType}`)
            }

            return selector

        })

        // Always filter on the id
        filterSelect.push({
            id: {
                $regex: ruleIdRegex.source
            }
        })

        const rules = await stubHelper.getQueryResultAsList({
            selector: {
                $and: filterSelect
            }
        })

        return rules
    }


    /**
    *   Requests to access some patient data
    *   PUBLIC
    */
    async getPatientAccess(stubHelper, args) {
        const decodedToken = await decodeToken(stubHelper, args[0])
        const user = new User(decodedToken)
        const accessRequest = await Helpers.checkArgs(args[1], Yup.object().shape(accessRequestShape).required())
        this.logger.info(`Requesting access: ${JSON.stringify(accessRequest)}`)

        const target = accessRequest
        const requestor = {
            individualId: user.individualId,
            groupIds: user.groupIds,
            orgId: user.orgId,
            departmentIds: user.departmentIds
        }

        // The requestor must match the grantee
        // The target must match the grantor
        const query = getQueryForAccessRequest(stubHelper, requestor, target)
        const rules = await stubHelper.getQueryResultAsList(query)

        this.logger.debug(`Rules are: ${JSON.stringify(rules)}`)

        // Returns true if some fetched rule applies to that attribute
        const ruleExistsForAttr = (targetAttr) => rules.some(
            (rule) => rule.access.some(
                (ruleAttr) => ruleAttr.name === targetAttr.name
            )
        )

        // Assume access granted unless otherwise stated.
        let accessGranted = true
        if (rules.some((rule) => rule.action === 'DENY')) {
            // If any selected rule is a deny, then access is not granted.
            accessGranted = false
        } else {
            // For each attribute for which the requestor asked access, we must confirm he has access
            accessGranted = target.access.every(ruleExistsForAttr)
        }

        const accessLog = {
            id: `ACCESS_REQUEST_${stubHelper.generateUUID('ACCESS_REQUEST')}`,
            accessGranted,
            timestamp: stubHelper.getTxDate().toISOString(),
            requestor,
            target
        }
        // Fehlerhafter Logoutput bei Aufruf mit QUERY?
        this.logger.info(`Going to write accessLog: ${JSON.stringify(accessLog)}`)
        // Save the request
        await stubHelper.putState(accessLog.id, accessLog)

        this.logger.info(`Access granted: ${accessGranted}`)

        return {
            accessGranted,
            access: target.access
        }
    }

    async getPatientAccessAll(stubHelper, args) {
        const decodedToken = await decodeToken(stubHelper, args[0])
        const user = new User(decodedToken)

        const {access, ...accessRequestWithoutAttributes} = accessRequestShape
        const accessRequest = await Helpers.checkArgs(args[1], Yup.object().shape(accessRequestWithoutAttributes).required())
        this.logger.info(`Requesting access: ${JSON.stringify(accessRequest)}`)

        const target = accessRequest
        const requestor = {
            individualId: user.individualId,
            groupIds: user.groupIds,
            orgId: user.orgId,
            departmentIds: user.departmentIds
        }

        const query = getQueryForAccessRequest(stubHelper, requestor, target)
        const rules = await stubHelper.getQueryResultAsList(query)

        // Get denied attributes
        let deniedAttributes = rules
            .filter((rule) => rule.action === 'DENY')
            .map((rule) => rule.access)
        deniedAttributes = [].concat.apply([], deniedAttributes) // Flatten
        deniedAttributes = removeDuplicates(deniedAttributes, 'name')

        // Get allowed attributes
        let allowedAttributes = rules
            .filter((rule) => rule.action === 'ALLOW')
            .map((rule) => rule.access)

        allowedAttributes = [].concat.apply([], allowedAttributes) // Flatten
        allowedAttributes = removeDuplicates(allowedAttributes, 'name')

        const isDeniedAttr = (attr) => deniedAttributes.some((deniedAttr) => attr.name === deniedAttr.name)

        this.logger.info(`Rules for query: ${JSON.stringify(rules)}`)
        const allowedNotDeniedAttributes = allowedAttributes
            .filter((attr) => !isDeniedAttr(attr))
        this.logger.info(
            `Getting allowedNotDeniedAttributes: ${JSON.stringify(allowedNotDeniedAttributes)}`
        )
        // Get rules with allow and have attributes
        let relevantRules = rules
            .filter((rule) => rule.action === 'ALLOW')
            .filter((rule) =>
                rule.access.some((prop) => allowedNotDeniedAttributes.includes(prop))
            )
        let relevantRuleIds = []
        relevantRules.forEach((rule) => relevantRuleIds.push(rule.id))
        this.logger.info(`Relevant ruleIDs: ${JSON.stringify(relevantRuleIds)}`)
        const accessGranted = allowedNotDeniedAttributes.length > 0

        // Get all attributes that give access to this
        const accessLog = {
            id: `ACCESS_REQUEST_${stubHelper.generateUUID('ACCESS_REQUEST')}`,
            accessGranted,
            timestamp: stubHelper.getTxDate().toISOString(),
            requestor,
            target: {
                ...target,
                access: allowedNotDeniedAttributes
            }
        }

        this.logger.info(JSON.stringify(accessLog))
        // Save the request
        await stubHelper.putState(accessLog.id, accessLog)

        this.logger.info(`Access granted: ${accessGranted}`)

        return {
            accessGranted,
            access: allowedNotDeniedAttributes,
            ruleIds: relevantRuleIds
        }

    }

    /**
    *   Request the access log for who asked permissions
    *   PUBLIC
    */
    async getAccessLog(stubHelper, args) {
        const {filters} = await Helpers.checkArgs(args[0] || {filters: []}, Yup.object().shape(accessLogFiltersShape).required())

        this.logger.info('Requesting access logs')

        const filterSelect = filters.map((f) => {
            const selector = {}
            selector[f.key] = {
                $eq: f.value
            }
            return selector
        })

        // Always filter on the id
        filterSelect.push({
            id: {
                $regex: '^ACCESS_REQUEST_.+'
            }
        })

        const accessLogs = await stubHelper.getQueryResultAsList({
            selector: {
                $and: filterSelect
            }
        })

        this.logger.info(`Access logs: ${JSON.stringify(accessLogs)}`)

        return accessLogs
    }

    /**
    *   Resets the state
    *   DEV ONLY, PUBLIC
    */
    async reset(stubHelper) {
        await stubHelper.deleteAllByQuery({
            selector: {
                id: {
                    $exists: true
                }
            },
            fields: ['id']
        })

        this.logger.info('State is reset')

        return {success: 'success'}
    }

    /**
    *   Creates a proof
    *   PERMISSIONED
    */
    async createProof(stubHelper, args) {
        let proof = await Helpers.checkArgs(args[0], Yup.object().shape(createProofShape).required())

        // Make proof for user
        this.logger.info(`Created proof for request: ${proof.serumsId}`)

        proof.id = `PROOF_${stubHelper.generateUUID('PROOF')}`

        let { hospitalIds, date, ...topLevelProperties} = proof

        let proofToCreate = {
            ...topLevelProperties,
            request_received: {
                hospitalIds,
                date
            }
        }

        await stubHelper.putState(proof.id, proofToCreate)
        this.logger.info(`Created proof ${proof.id}`)

        return {proofId: proof.id}
    }

    /**
    *   Updates an existing proof
    *   PERMISSIONED
    */
    async updateProof(stubHelper, args) {
        let proof = await Helpers.checkArgs(args[0], Yup.object().shape(updateProofShape).required())

        const proofId = await Helpers.checkArgs(args[1], proofIdShape.ensure())

        const previousProof = await stubHelper.getStateAsObject(proofId)
        if (!previousProof) {
            throw new ChaincodeError(`Proof with id ${proofId} does not exist.`, 404)
        }

        delete proof.proofId
        let { type, ...restOfProof} = proof
        delete restOfProof.id
        let { content, ...stepProperties} = restOfProof

        let proofToUpdate

        if (type != 'encryption') {
            let a = previousProof[type]
            if (a != undefined) {
                a.push(
                    {
                        ...stepProperties,
                        ...content
                    }
                )

                proofToUpdate = {
                    ...previousProof,
                    [type]: a
                }
            } else {
                proofToUpdate = {
                    ...previousProof,
                    [type]: [
                        {
                            ...stepProperties,
                            ...content
                        }
                    ]
                }
            }
        } else {
            proofToUpdate = {
                ...previousProof,
                [type]: {
                    ...stepProperties,
                    ...content
                }
            }
        }



        await stubHelper.putState(proofId, proofToUpdate)
        this.logger.info(`Updated rule ${proofId}`)

        return {proofId}
    }

    /**
    *   Get proof
    *   PUBLIC
    */
    async getProof(stubHelper, args) {
        this.logger.info('Getting Proof')
        const proofId = args[0]

        const filter = [
            {
                id: {
                    $eq: proofId
                }
            }
        ]

        const proof = await stubHelper.getQueryResultAsList({
            selector: {
                $and: filter
            }
        })

        return proof
    }

}

module.exports = SerumsChaincode
