const {
    ChaincodeMockStub,
    Transform
} = require("@theledger/fabric-mock-stub")

const SerumsChaincode = require('../serums')
const Rule = require('./testRules')
const Proof = require('./testProofs')
const PatientAccessRequest = require('./testRequest')

const getToken = require('./getToken')

// You always need your chaincode so it knows which chaincode to invoke on
const chaincode = new SerumsChaincode('info')

let currentTxId = 0

function nextTxId() {
    currentTxId += 1
    return `tx${currentTxId}`
}


async function testAccessLogs(mockStub, pr, expectAccessGranted) {
    const response = await mockStub.mockInvoke(
        nextTxId(),
        [
            'getAccessLog',
            JSON.stringify({
                filters: [
                    {
                        filterType: 'SIMPLE',
                        key: 'target.patientId',
                        value: pr.request.target.patientId
                    }
                ]
            })
        ]
    )

    expect(response.status).toBe(200)
    const payload = Transform.bufferToObject(response.payload)
    expect(
        payload.every(
            (accessLog) => new Date(accessLog.timestamp).getTime() <= new Date().getTime()
        )
    ).toBe(true)

    payload.forEach((accessLog) => {
        delete accessLog.timestamp
        delete accessLog.id
    })

    expect(payload).toEqual(expect.arrayContaining([
        {...pr.request, accessGranted: expectAccessGranted}
    ]))

    return payload
}

describe('Test serums chaincode', () => {
    // Tx1
    it('Should init without issues', async() => {
        const mockStub = new ChaincodeMockStub('MyMockStub', chaincode)
        // Your test code
        const response = await mockStub.mockInit(nextTxId(), [])

        expect(response.status).toBe(200)
    })

    // Tx2
    it('Should be able to ping', async() => {
        const mockStub = new ChaincodeMockStub('MyMockStub', chaincode)
        const response = await mockStub.mockInvoke(nextTxId(), ['ping'])

        expect(response.status).toBe(200)

        expect(Transform.bufferToObject(response.payload)).toEqual({
            ping: 'pong'
        })
    })

    // REQUEST_1
    it('Should be able to create a rule', async() => {
        const mockStub = new ChaincodeMockStub('MyMockStub', chaincode)
        const rule = new Rule()
            .allowAll('DOCTOR')
            .from('HOSPITAL_1')
            .toAccess(['EYE', 'BRAIN', 'ARM'])
            .of('PATIENT_1')
            .from('HOSPITAL_1')

        await rule.createRule(mockStub, true)
    })

    // REQUEST_2 + tx3
    it('Should be able to get a rule', async() => {
        const mockStub = new ChaincodeMockStub('MyMockStub', chaincode)
        const rule = new Rule()
            .allowAll('DOCTOR')
            .from('HOSPITAL_1')
            .toAccess(['EYE', 'BRAIN', 'ARM'])
            .of('PATIENT_1')
            .from('HOSPITAL_1')

        const ruleId = await rule.createRule(mockStub, true)


        const response = await mockStub.mockInvoke(
            nextTxId(),
            ['getRules']
        )
        expect(response.status).toBe(200)
        const payload = Transform.bufferToObject(response.payload)
        expect(payload).toEqual([{id: ruleId, ...rule.getFullRule()}])
    })

    // REQUEST_3 + tx4
    it('Should be able to get a rule with a filter', async() => {
        const mockStub = new ChaincodeMockStub('MyMockStub', chaincode)
        const rule = new Rule()
            .allowAll('DOCTOR')
            .from('HOSPITAL_1')
            .toAccess(['EYE', 'BRAIN', 'ARM'])
            .of('PATIENT_1')
            .from('HOSPITAL_1')

        const ruleId = await rule.createRule(mockStub, true)

        const response = await mockStub.mockInvoke(
            nextTxId(),
            [
                'getRules',
                {
                    filters: [{
                        filterType: 'SIMPLE',
                        key: 'grantor.type',
                        value: rule.rule.grantor.type
                    }]
                }
            ]
        )
        expect(response.status).toBe(200)
        const payload = Transform.bufferToObject(response.payload)
        expect(payload).toEqual([{id: ruleId, ...rule.getFullRule()}])
    })

    // REQUEST_4 + REQUEST_5 + tx5
    it('Should be able to update the rule', async() => {
        const mockStub = new ChaincodeMockStub('MyMockStub', chaincode)
        const rule = new Rule()
            .allowAll('DOCTOR')
            .from('HOSPITAL_1')
            .toAccess(['EYE', 'BRAIN', 'ARM'])
            .of('PATIENT_1')
            .from('HOSPITAL_1')

        const ruleId = await rule.createRule(mockStub, true)

        const updatedRule = new Rule()
            .allowAll('DOCTOR')
            .from('HOSPITAL_2')
            .toAccess(['EYE', 'BRAIN', 'ARM'])
            .of('PATIENT_1')
            .from('HOSPITAL_1')

        await updatedRule.updateRule(mockStub, ruleId, true)

        // Get rule and check change happened
        const response = await mockStub.mockInvoke(
            nextTxId(),
            ['getRules']
        )
        expect(response.status).toBe(200)
        const payload = Transform.bufferToObject(response.payload)
        expect(payload).toEqual([{id: ruleId, ...updatedRule.getFullRule()}])
    })

    // REQUEST_6 + REQUEST_7
    it('Should not be able to update the rule if not the owner', async() => {
        const mockStub = new ChaincodeMockStub('MyMockStub', chaincode)
        const rule = new Rule()
            .allowAll('DOCTOR')
            .from('HOSPITAL_1')
            .toAccess(['EYE', 'BRAIN', 'ARM'])
            .of('PATIENT_1')
            .from('HOSPITAL_1')

        const ruleId = await rule.createRule(mockStub, true)

        const updatedRule = new Rule()
            .allowAll('DOCTOR')
            .from('HOSPITAL_1')
            .toAccess(['EYE', 'BRAIN', 'ARM'])
            .of('PATIENT_1')
            .from('HOSPITAL_1')
            .asAnotherUser('PATIENT_3')
            .ofAnotherOrganisation('HOSPITAL_3')

        await updatedRule.updateRule(mockStub, ruleId, false)
    })

    // REQUEST_8 + REQUEST_9
    it('Should be able to update the rule to another grantor if I am an admin', async() => {
        const mockStub = new ChaincodeMockStub('MyMockStub', chaincode)
        const rule = new Rule()
            .allowAll('DOCTOR')
            .from('HOSPITAL_1')
            .toAccess(['EYE', 'BRAIN', 'ARM'])
            .of('PATIENT_1')
            .from('HOSPITAL_1')

        const ruleId = await rule.createRule(mockStub, true)

        const updatedRule = new Rule()
            .allowAll('DOCTOR')
            .from('HOSPITAL_1')
            .toAccess(['EYE', 'BRAIN', 'ARM'])
            .of('PATIENT_2')
            .from('HOSPITAL_1')
            .withGroups(['HOSPITAL_ADMIN'])

        await updatedRule.updateRule(mockStub, ruleId, true)
    })

    // REQUEST_10 + REQUEST_11
    it('Should not be able to update the rule to another grantor if I am not an admin', async() => {
        const mockStub = new ChaincodeMockStub('MyMockStub', chaincode)
        const rule = new Rule()
            .allowAll('DOCTOR')
            .from('HOSPITAL_1')
            .toAccess(['EYE', 'BRAIN', 'ARM'])
            .of('PATIENT_1')
            .from('HOSPITAL_1')

        const ruleId = await rule.createRule(mockStub, true)

        const updatedRule = new Rule()
            .allowAll('DOCTOR')
            .from('HOSPITAL_1')
            .toAccess(['EYE', 'BRAIN', 'ARM'])
            .of('PATIENT_2')
            .from('HOSPITAL_1')
            .withGroups([])

        await updatedRule.updateRule(mockStub, ruleId, false)
    })

    // REQUEST_12 + REQUEST_13
    it('Should not be able to update the rule to another grantor if I am an admin of another hospital', async() => {
        const mockStub = new ChaincodeMockStub('MyMockStub', chaincode)
        const rule = new Rule()
            .allowAll('DOCTOR')
            .from('HOSPITAL_1')
            .toAccess(['EYE', 'BRAIN', 'ARM'])
            .of('PATIENT_1')
            .from('HOSPITAL_1')

        const ruleId = await rule.createRule(mockStub, true)

        const updatedRule = new Rule()
            .allowAll('DOCTOR')
            .from('HOSPITAL_1')
            .toAccess(['EYE', 'BRAIN', 'ARM'])
            .of('PATIENT_2')
            .from('HOSPITAL_1')
            .withGroups(['HOSPITAL_ADMIN'])
            .ofAnotherOrganisation('HOSPITAL_2')

        await updatedRule.updateRule(mockStub, ruleId, false)
    })

    // REQUEST_14 + REQUEST_15
    it('Should not be able to update the rule to another grantor and hospital if I am an admin', async() => {
        const mockStub = new ChaincodeMockStub('MyMockStub', chaincode)
        const rule = new Rule()
            .allowAll('DOCTOR')
            .from('HOSPITAL_1')
            .toAccess(['EYE', 'BRAIN', 'ARM'])
            .of('PATIENT_1')
            .from('HOSPITAL_1')

        const ruleId = await rule.createRule(mockStub, true)

        const updatedRule = new Rule()
            .allowAll('DOCTOR')
            .from('HOSPITAL_1')
            .toAccess(['EYE', 'BRAIN', 'ARM'])
            .of('PATIENT_2')
            .from('HOSPITAL_2')
            .withGroups(['HOSPITAL_ADMIN'])

        await updatedRule.updateRule(mockStub, ruleId, false)
    })

    // REQUEST_16
    it('Should not be able to create a group rule if not an admin', async() => {
        const mockStub = new ChaincodeMockStub('MyMockStub', chaincode)
        const rule = new Rule()
            .allow('DOCTOR_1')
            .from('HOSPITAL_2')
            .toAccess(['EYE', 'BRAIN', 'ARM'])
            .ofAllPatientsOf('HOSPITAL_1')
            .asUser('DOCTOR_FROM_HOS_1')
            .asAnotherUser('DOCTOR_FROM_HOS_5')
            .ofAnotherOrganisation('HOSPITAL_5')
            .withGroups(['DOCTOR'])

        await rule.createRule(mockStub, false)
    })

    // REQUEST_17
    it('Should be able to create a rule that applies to another patient as an admin', async() => {
        const mockStub = new ChaincodeMockStub('MyMockStub', chaincode)
        const rule = new Rule()
            .allow('DOCTOR_1')
            .from('HOSPITAL_1')
            .toAccess(['EYE', 'BRAIN', 'ARM'])
            .of('PATIENT_1')
            .from('HOSPITAL_1')
            .asAnotherUser('ADMIN_HOS_1')
            .withGroups(['HOSPITAL_ADMIN'])

        await rule.createRule(mockStub, true)
    })

    // REQUEST_18
    it('Should not be able to create a rule that applies to another patient if not an admin', async() => {
        const mockStub = new ChaincodeMockStub('MyMockStub', chaincode)
        const rule = new Rule()
            .allow('DOCTOR_1')
            .from('HOSPITAL_1')
            .toAccess(['EYE', 'BRAIN', 'ARM'])
            .of('PATIENT_1')
            .from('HOSPITAL_1')
            .asAnotherUser('REGULAR_USER_HOS_1')
            .withGroups([])

        console.log(rule.rule)

        await rule.createRule(mockStub, false)
    })

    // REQUEST_19
    it('Should not be able to update a non existent rule', async() => {
        const mockStub = new ChaincodeMockStub('MyMockStub', chaincode)
        // Update rule
        const rule = new Rule()
            .allowAll('DOCTOR')
            .from('HOSPITAL_1')
            .toAccess(['EYE', 'BRAIN', 'ARM'])
            .of('PATIENT_1')
            .from('HOSPITAL_1')

        const error = await rule.updateRule(
            mockStub,
            'RULE_NOT_EXISTING',
            false
        )

        expect(error.message).toBe('Rule with id RULE_NOT_EXISTING does not exist.')
    })

    // REQUEST_20
    it('Should not be able to add an expired rule', async() => {
        const mockStub = new ChaincodeMockStub('MyMockStub', chaincode)
        // This one fails because expire is in the past
        const rule = new Rule()
            .allowAll('DOCTOR')
            .from('HOSPITAL_1')
            .toAccess(['EYE', 'BRAIN', 'ARM'])
            .of('PATIENT_1')
            .from('HOSPITAL_1')
            .makeExpired()

        const error = await rule.createRule(mockStub, false)
        expect(error.message).toBe('expires is in the past')
    })

    // REQUEST_21 + REQUEST_1
    it('Should work with individual granting to an individual', async() => {
        const mockStub = new ChaincodeMockStub('MyMockStub', chaincode)

        const rule = new Rule()
            .allow('DOCTOR_1')
            .from('HOSPITAL_1')
            .toAccess(['EYE', 'BRAIN', 'ARM'])
            .of('PATIENT_1')
            .from('HOSPITAL_1')

        await rule.createRule(mockStub, true)

        const pr = new PatientAccessRequest()
            .user('DOCTOR_1')
            .withGroups(['DOCTOR'])
            .from('HOSPITAL_1')
            .wantsToAccess(['EYE'])
            .ofPatient('PATIENT_1')
            .from('HOSPITAL_1')

        await pr.makeRequest(mockStub, true)
    })

    // REQUEST_22 + REQUEST_2
    it('Should work with an individual granting to a group', async() => {
        const mockStub = new ChaincodeMockStub('MyMockStub', chaincode)
        const rule = new Rule()
            .allowAll('DOCTOR')
            .from('HOSPITAL_1')
            .toAccess(['EYE', 'BRAIN', 'ARM'])
            .of('PATIENT_1')
            .from('HOSPITAL_1')

        await rule.createRule(mockStub, true)

        const pr = new PatientAccessRequest()
            .user('DOCTOR_1')
            .withGroups(['DOCTOR'])
            .from('HOSPITAL_1')
            .wantsToAccess(['EYE'])
            .ofPatient('PATIENT_1')
            .from('HOSPITAL_1')

        await pr.makeRequest(mockStub, true)
    })

    // REQUEST_22 + REQUEST_2
    it('Should work with an individual granting to a department', async() => {
        const mockStub = new ChaincodeMockStub('MyMockStub', chaincode)
        const rule = new Rule()
            .allowDepartment('DEP_1234')
            .from('HOSPITAL_1')
            .toAccess(['EYE', 'BRAIN', 'ARM'])
            .of('PATIENT_1')
            .from('HOSPITAL_1')

        await rule.createRule(mockStub, true)

        const pr = new PatientAccessRequest()
            .user('DOCTOR_1')
            .withDepartments(['DEP_1234'])
            .from('HOSPITAL_1')
            .wantsToAccess(['EYE'])
            .ofPatient('PATIENT_1')
            .from('HOSPITAL_1')

        await pr.makeRequest(mockStub, true)
    })

    // REQUEST_23
    it('Should work with a hospital granting to an individual', async() => {
        const mockStub = new ChaincodeMockStub('MyMockStub', chaincode)
        const rule = new Rule()
            .allow('DOCTOR_1')
            .from('HOSPITAL_2')
            .toAccess(['EYE', 'BRAIN', 'ARM'])
            .ofAllPatientsOf('HOSPITAL_1')
            .asUser('DOCTOR_FROM_HOS_1')

        await rule.createRule(mockStub, true)

        const pr = new PatientAccessRequest()
            .user('DOCTOR_1')
            .withGroups(['DOCTOR'])
            .from('HOSPITAL_2')
            .wantsToAccess(['EYE'])
            .ofPatient('PATIENT_1')
            .from('HOSPITAL_1')

        await pr.makeRequest(mockStub, true)
    })

    // REQUEST_24
    it('Should work with a hospital granting to a group', async() => {
        const mockStub = new ChaincodeMockStub('MyMockStub', chaincode)
        const rule = new Rule()
            .allowAll('DOCTOR')
            .from('HOSPITAL_1')
            .toAccess(['EYE', 'BRAIN', 'ARM'])
            .ofAllPatientsOf('HOSPITAL_1')
            .asUser('DOCTOR_FROM_HOS_1')

        await rule.createRule(mockStub, true)

        const pr = new PatientAccessRequest()
            .user('DOCTOR_1')
            .withGroups(['DOCTOR'])
            .from('HOSPITAL_1')
            .wantsToAccess(['EYE'])
            .ofPatient('PATIENT_1')
            .from('HOSPITAL_1')

        await pr.makeRequest(mockStub, true)
    })

    // REQUEST_25
    it('Should not work with an attribute that does not exist', async() => {
        const mockStub = new ChaincodeMockStub('MyMockStub', chaincode)
        const rule = new Rule()
            .allow('DOCTOR_1')
            .from('HOSPITAL_2')
            .toAccess(['EYE', 'BRAIN', 'ARM'])
            .ofAllPatientsOf('HOSPITAL_1')
            .asUser('DOCTOR_FROM_HOS_1')

        await rule.createRule(mockStub, true)

        const pr = new PatientAccessRequest()
            .user('DOCTOR_1')
            .withGroups(['DOCTOR'])
            .from('HOSPITAL_2')
            .wantsToAccess(['THIS_DOES_NOT_EXIST'])
            .ofPatient('PATIENT_1')
            .from('HOSPITAL_1')

        await pr.makeRequest(mockStub, false)
    })

    // REQUEST_27 + REQUEST_29 + REQUEST_4 +
    it('Should not work if there is a DENY overriding an ALLOW', async() => {
        const mockStub = new ChaincodeMockStub('MyMockStub', chaincode)
        const rule = new Rule()
            .allow('DOCTOR_1')
            .from('HOSPITAL_2')
            .toAccess(['EYE', 'BRAIN', 'ARM'])
            .ofAllPatientsOf('HOSPITAL_1')
            .asUser('DOCTOR_FROM_HOS_1')
        await rule.createRule(mockStub, true)

        const denyRule = new Rule()
            .deny('DOCTOR_1')
            .from('HOSPITAL_2')
            .toAccess(['EYE'])
            .ofAllPatientsOf('HOSPITAL_1')
            .asUser('DOCTOR_FROM_HOS_1')

        await denyRule.createRule(mockStub, true)

        let pr = new PatientAccessRequest()
            .user('DOCTOR_1')
            .withGroups(['DOCTOR'])
            .from('HOSPITAL_2')
            .wantsToAccess(['EYE'])
            .ofPatient('PATIENT_1')
            .from('HOSPITAL_1')

        await pr.makeRequest(mockStub, false)

        pr = new PatientAccessRequest()
            .user('DOCTOR_1')
            .withGroups(['DOCTOR'])
            .from('HOSPITAL_2')
            .wantsToAccess(['BRAIN'])
            .ofPatient('PATIENT_1')
            .from('HOSPITAL_1')

        await pr.makeRequest(mockStub, true)
    })

    it('Should work when attributes are spread over multiple rules', async() => {
        const mockStub = new ChaincodeMockStub('MyMockStub', chaincode)
        const rule = new Rule()
            .allow('DOCTOR_1')
            .from('HOSPITAL_2')
            .toAccess(['EYE'])
            .ofAllPatientsOf('HOSPITAL_1')
            .asUser('DOCTOR_FROM_HOS_1')

        await rule.createRule(mockStub, true)

        const secondRule = new Rule()
            .allow('DOCTOR_1')
            .from('HOSPITAL_2')
            .toAccess(['BRAIN'])
            .ofAllPatientsOf('HOSPITAL_1')
            .asUser('DOCTOR_FROM_HOS_1')

        await secondRule.createRule(mockStub, true)

        const pr = new PatientAccessRequest()
            .user('DOCTOR_1')
            .withGroups(['DOCTOR'])
            .from('HOSPITAL_2')
            .wantsToAccess(['EYE', 'BRAIN'])
            .ofPatient('PATIENT_1')
            .from('HOSPITAL_1')

        await pr.makeRequest(mockStub, true)
    })

    it('Should be able to retrieve the attributes to which I have access, and make an access request', async() => {
        const mockStub = new ChaincodeMockStub('MyMockStub', chaincode)
        const rule = new Rule()
            .allowAll('DOCTOR')
            .from('HOSPITAL_1')
            .toAccess(['EYE', 'BRAIN', 'ARM'])
            .of('PATIENT_1')
            .from('HOSPITAL_1')

        await rule.createRule(mockStub)

        const secondRule = new Rule()
            .deny('DOCTOR_1')
            .from('HOSPITAL_1')
            .toAccess(['EYE'])
            .of('PATIENT_1')
            .from('HOSPITAL_1')

        await secondRule.createRule(mockStub)

        const pr = new PatientAccessRequest()
            .user('DOCTOR_1')
            .withGroups(['DOCTOR'])
            .from('HOSPITAL_1')
            .wantsToAccess('all')
            .ofPatient('PATIENT_1')
            .from('HOSPITAL_1')

        const payload = await pr.makeRequest(mockStub, true)
        expect(payload.access).toEqual(expect.arrayContaining([
            {name: 'BRAIN'},
            {name: 'ARM'}
        ]))
        expect(payload.access).toEqual(expect.not.arrayContaining([
            {name: 'EYE'}
        ]))

        console.log(`Payload: ${JSON.stringify(payload)}`)
        // The request we made was only for the attributes we can access to
        pr.request.target.access = payload.access
        const accessLogs = await testAccessLogs(mockStub, pr, true)
        expect(accessLogs).toHaveLength(1)
    })

    it('Should be able to get the access logs', async() => {
        const mockStub = new ChaincodeMockStub('MyMockStub', chaincode)
        const pr = new PatientAccessRequest()
            .user('DOCTOR_DENIED')
            .withGroups(['DOCTOR'])
            .from('HOSPITAL_1')
            .wantsToAccess(['EYE', 'BRAIN'])
            .ofPatient('PATIENT_1')
            .from('HOSPITAL_1')
        await pr.makeRequest(mockStub, false)

        const payload = await testAccessLogs(mockStub, pr, false)

        expect(payload).toHaveLength(1)
    })

    it('Should be able to delete the rule', async() => {
        const mockStub = new ChaincodeMockStub('MyMockStub', chaincode)
        const rule = new Rule()
            .allowAll('DOCTOR')
            .from('HOSPITAL_1')
            .toAccess(['EYE', 'BRAIN', 'ARM'])
            .of('PATIENT_1')
            .from('HOSPITAL_1')

        const ruleId = await rule.createRule(mockStub)

        const token = getToken({
            id: 'PATIENT_1',
            orgId: 'HOSPITAL_1',
            groupIds: [] // groups
        })

        let response = await mockStub.mockInvoke(
            nextTxId(),
            ['deleteRule', token, ruleId]
        )
        expect(response.status).toBe(200)

        // Get the rules and check it does not contain our rule
        response = await mockStub.mockInvoke(
            nextTxId(),
            ['getRules']
        )
        expect(response.status).toBe(200)
        payload = Transform.bufferToObject(response.payload)
        expect(payload.some((rule) => rule.id === ruleId)).toEqual(false)
    })

    it('Should not be able to delete the rule if not owner of rule', async() => {
        const mockStub = new ChaincodeMockStub('MyMockStub', chaincode)
        const rule = new Rule()
            .allowAll('DOCTOR')
            .from('HOSPITAL_1')
            .toAccess(['EYE', 'BRAIN', 'ARM'])
            .of('PATIENT_1')
            .from('HOSPITAL_1')

        const ruleId = await rule.createRule(mockStub)

        const token = getToken({
            id: 'PATIENT_2',
            orgId: 'HOSPITAL_1',
            groupIds: [] // groups
        })

        const response = await mockStub.mockInvoke(
            nextTxId(),
            ['deleteRule', token, ruleId]
        )
        expect(response.status).toBe(500)
    })

    // Provenance

    it('Should be able to create a proof', async() => {
        const mockStub = new ChaincodeMockStub('MyMockStub', chaincode)

        const proof = new Proof()

        await proof.createProof(mockStub, true)
    })

    it('Should be able to get a proof', async() => {
        const mockStub = new ChaincodeMockStub('MyMockStub', chaincode)

        const proof = new Proof()
        const proofId = await proof.createProof(mockStub, true)

        const response = await mockStub.mockInvoke(
            nextTxId(),
            ['getProof', proofId]
        )

        expect(response.status).toBe(200)

        const payload = Transform.bufferToObject(response.payload)
        const proofvalue = [{id: proofId, ...proof.getFullProof()}]
        expect(payload).toEqual(proofvalue)
    })

    it('Should be able to update the proof', async() => {
        const mockStub = new ChaincodeMockStub('MyMockStub', chaincode)

        const proof = new Proof()
        const proofId = await proof.createProof(mockStub, true)

        const updatedproof = new Proof()
        await updatedproof
            .type_data_filled()
            .updateProof(mockStub, proofId, true)

        const response = await mockStub.mockInvoke(
            nextTxId(),
            ['getProof', proofId]
        )
        expect(response.status).toBe(200)

        const payload = Transform.bufferToObject(response.payload)
        const proofvalue = [{id: proofId, ...updatedproof.getFullProof()}]

        expect(payload).toEqual(proofvalue)
    })

    it('Should be able to update the proof, encryption step', async() => {
        const mockStub = new ChaincodeMockStub('MyMockStub', chaincode)

        const proof = new Proof()
        const proofId = await proof.createProof(mockStub, true)

        const updatedproof = new Proof()
        await updatedproof
            .type_encryption()
            .updateProof(mockStub, proofId, true)

        const response = await mockStub.mockInvoke(
            nextTxId(),
            ['getProof', proofId]
        )
        expect(response.status).toBe(200)

        const payload = Transform.bufferToObject(response.payload)
        const proofvalue = [{id: proofId, ...updatedproof.getFullProof()}]

        expect(payload).toEqual(proofvalue)
    })

    it('Should be able to update the proof, multiple hospital records', async() => {
        const mockStub = new ChaincodeMockStub('MyMockStub', chaincode)

        const proof = new Proof()
        const proofId = await proof.createProof(mockStub, true)

        const updatedproof = new Proof()
        await updatedproof
            .type_data_vault_created()
            .updateProof(mockStub, proofId, true)

        const updatedproof_second_hospital = new Proof()
        await updatedproof_second_hospital
                .type_data_vault_created('ZMC')
                .updateProof(mockStub, proofId, true)

        const response = await mockStub.mockInvoke(
            nextTxId(),
            ['getProof', proofId]
        )
        expect(response.status).toBe(200)

        const payload = Transform.bufferToObject(response.payload)


        const matchingProof  = updatedproof.getFullProof()
        matchingProof.data_vault_created.push(
            updatedproof_second_hospital.getFullProof().data_vault_created[0]
        )

        const proofvalue = [{id: proofId, ...matchingProof}]

        expect(payload).toEqual(proofvalue)
    })
})
